/***************************************************************************
 *   Copyright (C) 2005 by Christian Muehlhaeuser, Last.fm Ltd.            *
 *   chris@last.fm                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include <QtGui>
#include <QHttp>
#include <QFileInfo>
#include <QMovie>

#include "player.h"
#include "song.h"
#include "aboutdialog.h"
#include "urldialog.h"
#include "tagdialog.h"
#include "coverloader.h"
#include "imagefader.h"
#include "settings.h"

#ifdef WIN32
#include <windows.h>
#endif

bool handshaked = false;
bool minimode = false;
QString commandLine = "";
Song playingSong;


bool
transmitCli( QString parameter )
{
    QTcpSocket *socket = new QTcpSocket( 0 );
    socket->connectToHost( QHostAddress::LocalHost, 32213 );
    if ( socket->waitForConnected( 500 ) )
    {
        socket->write( parameter.toLocal8Bit(), parameter.length() );
        socket->flush();
        socket->close();
        return true;
    }

    return false;
}


#ifdef Q_WS_MAC
#include <ApplicationServices/ApplicationServices.h>

static pascal OSErr
getURLEvent( const AppleEvent *appEvent, AppleEvent *reply, long handlerRefcon )
{
    DescType type;
    Size size;

    char buf[1024];
    AEGetParamPtr( appEvent, keyDirectObject, typeChar, &type, &buf, 1023, &size );
    buf[ size ] = '\0';

    qDebug() << "Coming in:";
    qDebug() << buf;

    if ( handshaked )
        transmitCli( QString::fromUtf8( buf ).remove( 0, 9 ) );
    else
        commandLine = QString::fromUtf8( buf ).remove( 0, 9 );
}
#endif


Player::Player( QWidget *parent, QString cli, QString dataPath )
    : QMainWindow( parent )
    , ws( new WebserviceConnector() )
    , station( new StationDialog( this, ws ) )
    , playback( 0 )
    , m_timer( 0 )
    , coverBuffer( 0 )
    , animation( 0 )
    , m_proxyConnected( false )
{
    #ifdef Q_WS_MAC
    AEInstallEventHandler( 'GURL', 'GURL', NewAEEventHandlerUPP( getURLEvent ), 0, false );
    #endif

    setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );

    m_dataPath = QFileInfo( dataPath ).absolutePath() + "/";
    coverLoader = new CoverLoader( this, m_dataPath + "cache/" );

    #ifndef Q_WS_MAC
    if ( !cli.isEmpty() && cli.length() > 9 )
        commandLine = cli.remove( 0, 9 );
    #endif

    // check for another instance running
    if ( !commandLine.isEmpty() && transmitCli( commandLine ) )
        QApplication::quit();
    else
    {
        serverSocket = new QTcpServer( this );
        connect( serverSocket, SIGNAL( newConnection() ), this, SLOT( webConnect() ) );
        serverSocket->listen( QHostAddress::LocalHost, 32213 );

        ui.setupUi( this );

        QIcon icon( m_dataPath + "data/icon.png" );
        setWindowIcon( icon );

        menu = new QMenu( this );
        stationAction = new QAction( tr( "Change &Station..." ), this );
        urlAction = new QAction( tr( "Enter Station &Address..." ), this );
        recordAction = new QAction( tr( "Record to &Profile" ), this );
        discoveryAction = new QAction( tr( "&Discovery Mode" ), this );
        refreshAction = new QAction( tr( "&Refresh" ), this );
        configureAction = new QAction( tr( "&Settings..." ), this );
        externalPlayerAction = new QAction( tr( "Use &External Player" ), this );
        onTopAction = new QAction( tr( "Stay on &Top" ), this );
        aboutAction = new QAction( tr( "&About" ), this );

        #ifdef Q_WS_MAC
        stationAction->setShortcut( tr( "Ctrl+N" ) );
        urlAction->setShortcut( tr( "Ctrl+G" ) );
        refreshAction->setShortcut( tr( "Ctrl+R" ) );

        QMenu *fileMenu = menuBar()->addMenu( tr( "&File" ) );
        fileMenu->addAction( configureAction );
        fileMenu->addAction( aboutAction );

        QMenu *streamMenu = menuBar()->addMenu( tr( "&Controls" ) );
        streamMenu->addAction( stationAction );
        streamMenu->addAction( urlAction );
        streamMenu->addSeparator();
        streamMenu->addAction( recordAction );
        streamMenu->addAction( discoveryAction );
        streamMenu->addSeparator();
        streamMenu->addAction( refreshAction );
        #endif

        stationAction->setEnabled( false );
        urlAction->setEnabled( false );
        recordAction->setEnabled( false );
        discoveryAction->setEnabled( false );
        refreshAction->setEnabled( false );

        recordAction->setCheckable( true );
        recordAction->setChecked( true );
        discoveryAction->setCheckable( true );
        discoveryAction->setChecked( false );
        onTopAction->setCheckable( true );
        onTopAction->setChecked( false );
        externalPlayerAction->setCheckable( true );

        menu->addAction( stationAction );
        menu->addAction( urlAction );
        menu->addSeparator();
        menu->addAction( recordAction );
        menu->addAction( discoveryAction );
        menu->addAction( refreshAction );
        menu->addSeparator();
        menu->addAction( configureAction );
        menu->addAction( externalPlayerAction );
        menu->addAction( onTopAction );
        menu->addSeparator();
        menu->addAction( aboutAction );

        int externalSystem = -1;
        #ifdef WIN32
        externalSystem = 1;
        #endif
        #ifdef Q_WS_X11
        externalSystem = 2;
        #endif

        externalPlayerAction->setChecked( Settings::instance()->soundSystem() == externalSystem );

        init();

        connect( stationAction, SIGNAL( triggered() ), this, SLOT( openStationDialog() ) );
        connect( urlAction, SIGNAL( triggered() ), this, SLOT( openUrlDialog() ) );
        connect( recordAction, SIGNAL( toggled( bool ) ), this, SLOT( recordToProfile( bool ) ) );
        connect( discoveryAction, SIGNAL( toggled( bool ) ), this, SLOT( discoveryMode( bool ) ) );
        connect( refreshAction, SIGNAL( triggered() ), ws, SLOT( metaData() ) );
        connect( configureAction, SIGNAL( triggered() ), this, SLOT( openConfig() ) );
        connect( externalPlayerAction, SIGNAL( toggled( bool ) ), this, SLOT( useExternalPlayer( bool ) ) );
        connect( onTopAction, SIGNAL( toggled( bool ) ), this, SLOT( setStayOnTop( bool ) ) );
        connect( aboutAction, SIGNAL( triggered() ), this, SLOT( openAbout() ) );

        connect( ws, SIGNAL( handshakeResult( bool, bool, QUrl ) ), this, SLOT( handshakeResult( bool, bool, QUrl ) ) );
        connect( ws, SIGNAL( metaDataResult( Song, bool ) ), this, SLOT( metaDataResult( Song, bool ) ) );
        connect( ws, SIGNAL( skipResult() ), this, SLOT( skipResult() ) );
        connect( ws, SIGNAL( banResult() ), this, SLOT( banResult() ) );
        connect( ws, SIGNAL( changeStationPrepared() ), this, SLOT( changeStationPrepared() ) );
        connect( ws, SIGNAL( changeStationResult( bool ) ), this, SLOT( changeStationResult( bool ) ) );
        connect( ws, SIGNAL( actionStarted() ), this, SLOT( animationStart() ) );
        connect( ws, SIGNAL( actionFinished() ), this, SLOT( animationStop() ) );

        connect( Settings::instance(), SIGNAL( configChanged() ), this, SLOT( handshake() ) );

        Settings::instance()->setDataPath( m_dataPath );
        if ( Settings::instance()->positionX() > 0 && Settings::instance()->positionY() > 0 )
             move( Settings::instance()->positionX(), Settings::instance()->positionY() );

        if ( Settings::instance()->username().isEmpty() || Settings::instance()->password().isEmpty() )
        {
            Settings::instance()->setInitial( true );
            QApplication::setQuitOnLastWindowClosed( false );
            Settings::instance()->exec();
            QApplication::setQuitOnLastWindowClosed( true );
        }

        if ( Settings::instance()->proxyUsage() && !Settings::instance()->proxyHost().isEmpty() )
            qDebug( "Using proxy!" );

        handshake();
    }
}


Player::~Player()
{
    delete playback;
}


void
Player::init()
{
    if ( minimode )
    {
        fader = new ImageFader();
        loadDefaultCover();

        connect( coverLoader, SIGNAL( requestFinished( bool ) ), this, SLOT( coverLoaded( bool ) ) );
        connect( fader, SIGNAL( fadeProgress( const QPixmap & ) ), ui_mini.coverImage, SLOT( setPixmap( const QPixmap & ) ) );

        QPixmap image;
        image.load( m_dataPath + "data/artist.png" );
        ui_mini.artistImage->setPixmap( image );
        image.load( m_dataPath + "data/album.png" );
        ui_mini.albumImage->setPixmap( image );
        image.load( m_dataPath + "data/song.png" );
        ui_mini.songImage->setPixmap( image );
        image.load( m_dataPath + "data/volume_left.png" );
        ui_mini.volumeLow->setPixmap( image );
        image.load( m_dataPath + "data/volume_right.png" );
        ui_mini.volumeHigh->setPixmap( image );
        image.load( m_dataPath + "data/curve.png" );
        ui_mini.curveLabel->setPixmap( image );

        ui_mini.stopButton->setImages( m_dataPath, "data/buttons/play_enabled.png", "data/buttons/play_enabled.png", "data/buttons/play_hover.png", "data/buttons/stop_disabled.png" );
        ui_mini.skipButton->setImages( m_dataPath, "data/buttons/skip_enabled.png", "data/buttons/skip_down.png", "data/buttons/skip_hover.png", "data/buttons/skip_disabled.png" );
        ui_mini.loveButton->setImages( m_dataPath, "data/buttons/love_enabled.png", "data/buttons/love_down.png", "data/buttons/love_hover.png", "data/buttons/love_disabled.png" );
        ui_mini.banButton->setImages ( m_dataPath, "data/buttons/ban_enabled.png", "data/buttons/ban_down.png", "data/buttons/ban_hover.png", "data/buttons/ban_disabled.png" );
        ui_mini.menuButton->setImages( m_dataPath, "data/buttons/config_enabled.png", "data/buttons/config_down.png", "data/buttons/config_hover.png", "data/buttons/config_disabled.png" );

        ui_mini.tagButton->setImages ( m_dataPath, "data/buttons/tag_normal.png", "data/buttons/tag_down.png", "data/buttons/tag_hover.png", "data/buttons/tag_disabled.png" );
        ui_mini.blogButton->setImages( m_dataPath, "data/buttons/blog_normal.png", "data/buttons/blog_down.png", "data/buttons/blog_hover.png", "data/buttons/blog_disabled.png" );

        ui_mini.stationLabel->setText( "<font color='white'><b>Connecting to server...</b></font>" );

        ui_mini.volumeSlider->setValue( Settings::instance()->volume() );
        volumeChanged( Settings::instance()->volume() );

        ui_mini.skipButton->setEnabled( false );
        ui_mini.loveButton->setEnabled( false );
        ui_mini.banButton->setEnabled ( false );
        ui_mini.tagButton->setEnabled ( false );
        ui_mini.blogButton->setEnabled( false );

        connect( ui_mini.volumeSlider, SIGNAL( sliderMoved( int ) ), this, SLOT( volumeChanged( int ) ) );
        connect( ui_mini.skipButton, SIGNAL( clicked() ), this, SLOT( skipSong() ) );
        connect( ui_mini.loveButton, SIGNAL( clicked() ), this, SLOT( loveSong() ) );
        connect( ui_mini.banButton, SIGNAL( clicked() ), this, SLOT( banSong() ) );
        connect( ui_mini.stopButton, SIGNAL( clicked() ), this, SLOT( stopPlayback() ) );
        connect( ui_mini.menuButton, SIGNAL( clicked() ), this, SLOT( openMenu() ) );

        connect( ui_mini.coverImage, SIGNAL( clicked() ), this, SLOT( browseAlbum() ) );
        connect( ui_mini.artistLabel, SIGNAL( clicked() ), this, SLOT( browseArtist() ) );
        connect( ui_mini.albumLabel, SIGNAL( clicked() ), this, SLOT( browseAlbum() ) );
        connect( ui_mini.trackLabel, SIGNAL( clicked() ), this, SLOT( browseTrack() ) );

        connect( ui_mini.tagButton, SIGNAL( clicked() ), this, SLOT( openTagDialog() ) );
        connect( ui_mini.blogButton, SIGNAL( clicked() ), this, SLOT( openJournalMenu() ) );
    }
    else
    {
        fader = new ImageFader();
        loadDefaultCover();

        connect( coverLoader, SIGNAL( requestFinished( bool ) ), this, SLOT( coverLoaded( bool ) ) );
        connect( fader, SIGNAL( fadeProgress( const QPixmap & ) ), ui.coverImage, SLOT( setPixmap( const QPixmap & ) ) );

        QPixmap image;
        image.load( m_dataPath + "data/artist.png" );
        ui.artistImage->setPixmap( image );
        image.load( m_dataPath + "data/album.png" );
        ui.albumImage->setPixmap( image );
        image.load( m_dataPath + "data/song.png" );
        ui.songImage->setPixmap( image );
        image.load( m_dataPath + "data/volume_left.png" );
        ui.volumeLow->setPixmap( image );
        image.load( m_dataPath + "data/volume_right.png" );
        ui.volumeHigh->setPixmap( image );
        image.load( m_dataPath + "data/curve.png" );
        ui.curveLabel->setPixmap( image );

        ui.stopButton->setImages( m_dataPath, "data/buttons/play_enabled.png", "data/buttons/play_enabled.png", "data/buttons/play_hover.png", "data/buttons/stop_disabled.png" );
        ui.skipButton->setImages( m_dataPath, "data/buttons/skip_enabled.png", "data/buttons/skip_down.png", "data/buttons/skip_hover.png", "data/buttons/skip_disabled.png" );
        ui.loveButton->setImages( m_dataPath, "data/buttons/love_enabled.png", "data/buttons/love_down.png", "data/buttons/love_hover.png", "data/buttons/love_disabled.png" );
        ui.banButton->setImages ( m_dataPath, "data/buttons/ban_enabled.png", "data/buttons/ban_down.png", "data/buttons/ban_hover.png", "data/buttons/ban_disabled.png" );
        ui.menuButton->setImages( m_dataPath, "data/buttons/config_enabled.png", "data/buttons/config_down.png", "data/buttons/config_hover.png", "data/buttons/config_disabled.png" );

        ui.tagButton->setImages ( m_dataPath, "data/buttons/tag_normal.png", "data/buttons/tag_down.png", "data/buttons/tag_hover.png", "data/buttons/tag_disabled.png" );
        ui.blogButton->setImages( m_dataPath, "data/buttons/blog_normal.png", "data/buttons/blog_down.png", "data/buttons/blog_hover.png", "data/buttons/blog_disabled.png" );

        ui.stationLabel->setText( "<font color='white'><b>Connecting to server...</b></font>" );

        ui.volumeSlider->setValue( Settings::instance()->volume() );
        volumeChanged( Settings::instance()->volume() );

        ui.skipButton->setEnabled( false );
        ui.loveButton->setEnabled( false );
        ui.banButton->setEnabled ( false );
        ui.tagButton->setEnabled ( false );
        ui.blogButton->setEnabled( false );

        connect( ui.volumeSlider, SIGNAL( sliderMoved( int ) ), this, SLOT( volumeChanged( int ) ) );
        connect( ui.skipButton, SIGNAL( clicked() ), this, SLOT( skipSong() ) );
        connect( ui.loveButton, SIGNAL( clicked() ), this, SLOT( loveSong() ) );
        connect( ui.banButton, SIGNAL( clicked() ), this, SLOT( banSong() ) );
        connect( ui.stopButton, SIGNAL( clicked() ), this, SLOT( stopPlayback() ) );
        connect( ui.menuButton, SIGNAL( clicked() ), this, SLOT( openMenu() ) );

        connect( ui.coverImage, SIGNAL( clicked() ), this, SLOT( browseAlbum() ) );
        connect( ui.artistLabel, SIGNAL( clicked() ), this, SLOT( browseArtist() ) );
        connect( ui.albumLabel, SIGNAL( clicked() ), this, SLOT( browseAlbum() ) );
        connect( ui.trackLabel, SIGNAL( clicked() ), this, SLOT( browseTrack() ) );

        connect( ui.tagButton, SIGNAL( clicked() ), this, SLOT( openTagDialog() ) );
        connect( ui.blogButton, SIGNAL( clicked() ), this, SLOT( openJournalMenu() ) );
    }
}


void
Player::handshake()
{
    if ( playback != 0 )
    {
        delete playback;
        playbackFinished();
    }

    ws->handshake( Settings::instance()->username(), Settings::instance()->password() );
    playback = new Playback();

    connect( playback, SIGNAL( playbackStarting() ), this, SLOT( playbackStarting() ) );
    connect( playback, SIGNAL( playbackFinished() ), this, SLOT( playbackFinished() ) );
    connect( playback, SIGNAL( proxyClientConnected() ), this, SLOT( proxyClientConnected() ) );
    connect( playback, SIGNAL( proxyClientDisconnected() ), this, SLOT( proxyClientDisconnected() ) );
}


void
Player::switchMode()
{
}


void
Player::loadDefaultCover()
{
    QPixmap image;
    image.load( m_dataPath + "data/nocover.gif" );
    ui.coverImage->setPixmap( image );
}


void
Player::handshakeResult( bool connected, bool loginDone, const QUrl &streamUrl )
{
    handshaked = connected;
    if ( loginDone )
    {
        station->updateInformation();

        if ( !minimode )
            ui.stationLabel->setText( "<font color='white'><b>Please start a station!</b></font>" );

        stationAction->setEnabled( true );
        urlAction->setEnabled( true );
        recordAction->setEnabled( true );
        refreshAction->setEnabled( true );
        discoveryAction->setEnabled( ws->isSubscriber() );

        this->streamUrl = streamUrl;

        if ( !commandLine.isEmpty() && commandLine.length() > 9 )
        {
            ws->changeStation( commandLine );
            commandLine = "";
        }
        else
            openStationDialog();
    }
    else
    {
        if ( connected )
          QMessageBox::critical( 0, "Error", "Login failed! Please check your username and password!\n" );
        else
          QMessageBox::critical( 0, "Error", "Can't connect to server! Make sure your Internet connection is working and your proxy settings are correct!\n" );

        Settings::instance()->exec();
    }
}


void
Player::metaDataResult( const Song &song, bool discovery )
{
    if ( !playback->isPlaying() )
        return;

    playingSong.fetch( song );

    if ( ws->isSubscriber() )
        discoveryAction->setEnabled( discovery );

    if ( playingSong.duration() == 0 )
    {
        if ( minimode )
        {
            ui_mini.timeBar->setMaximum( 100 );
            ui_mini.timeBar->setValue( 100 );
        }
        else
        {
            ui.timeBar->setMaximum( 100 );
            ui.timeBar->setValue( 100 );
        }
    }
    else
    {
        if ( minimode )
        {
            ui_mini.timeBar->setMaximum( playingSong.duration() );
            ui_mini.timeBar->setValue( 0 );
        }
        else
        {
            ui.timeBar->setMaximum( playingSong.duration() );
            ui.timeBar->setValue( 0 );
        }
    }

    if ( m_timer != 0 )
    {
        m_timer->stop();
        delete m_timer;
    }

    m_timer = new QTimer( this );
    connect( m_timer, SIGNAL( timeout() ), this, SLOT( updateTimeBar() ) );
    m_timer->setInterval( 1000 );
    m_timer->start();

    if ( minimode )
    {
        ui_mini.tagButton->setEnabled ( true );
        ui_mini.blogButton->setEnabled( true );

        ui_mini.skipButton->setEnabled( true );
        ui_mini.loveButton->setEnabled( true );
        ui_mini.banButton->setEnabled ( true );
        ui_mini.wholeLabel->setText( playingSong.artist() + " " + playingSong.track() );
    }
    else
    {
        ui.tagButton->setEnabled ( true );
        ui.blogButton->setEnabled( true );

        ui.skipButton->setEnabled( true );
        ui.loveButton->setEnabled( true );
        ui.banButton->setEnabled ( true );
        ui.artistLabel->setText( playingSong.artist() );
        ui.albumLabel->setText( playingSong.album() );
        ui.trackLabel->setText( playingSong.track() );

        if ( Settings::instance()->debug() )
            ui.stationLabel->setText( "<font color='white'><b>Debug: " + playingSong.station() + "</b></font>" );
        else
            ui.stationLabel->setText( "<font color='white'><b>" + playingSong.station() + "</b></font>" );
    }

    if ( coverBuffer != 0 ) delete coverBuffer;
    coverBuffer = new QBuffer();

    if ( playingSong.cover().isEmpty() )
        loadDefaultCover();
    else
    {
        coverLoader->setHost( playingSong.cover().host() );
        if ( Settings::instance()->proxyUsage() && !Settings::instance()->proxyHost().isEmpty() )
            coverLoader->setProxy( Settings::instance()->proxyHost(), Settings::instance()->proxyPort(), Settings::instance()->proxyUsername(), Settings::instance()->proxyPassword() );

        if ( !playingSong.cover().encodedQuery().isEmpty() )
        {
            qDebug( QString( playingSong.cover().host() + playingSong.cover().path() + "?" + QString( playingSong.cover().encodedQuery() ) ).toLocal8Bit() );
            coverLoader->get( playingSong.cover().path() + "?" + QString( playingSong.cover().encodedQuery() ), coverBuffer );
        }
        else
        {
            qDebug( QString( playingSong.cover().host() + playingSong.cover().path() ).toLocal8Bit() );
            coverLoader->get( playingSong.cover().path(), coverBuffer );
        }
    }
}


void
Player::skipResult()
{
}


void
Player::banResult()
{
}


void
Player::updateTimeBar()
{
    playingSong.setProgress( playingSong.progress() + 1 );

    if ( minimode )
        ui_mini.timeBar->setValue( playingSong.progress() );
    else
        ui.timeBar->setValue( playingSong.progress() );
}


void
Player::animationStart()
{
    if ( !animation || animation->state() != QMovie::Running )
    {
        if ( animation ) delete animation;
        animation = new QMovie( m_dataPath + "data/progress_active.gif" );
        animation->setSpeed( 150 );

        if ( minimode )
            ui_mini.logoLabel->setMovie( animation );
        else
            ui.logoLabel->setMovie( animation );

        animation->start();
        connect( animation, SIGNAL( finished() ), this, SLOT( animationStart() ) );
    }
}


void
Player::animationStop()
{
    QPixmap image;
    image.load( m_dataPath + "data/progress_inactive.gif" );

    if ( minimode )
        ui_mini.logoLabel->setPixmap( image );
    else
        ui.logoLabel->setPixmap( image );

    if ( animation ) delete animation;
    animation = 0;
}


void
Player::coverLoaded( bool error )
{
    if ( !playback->isPlaying() )
        return;

    qDebug( "Cover returned!" );

    if ( !error && coverBuffer->size() > 0 )
    {
        qDebug( "Setting cover!" );
        QPixmap cover;
        cover.loadFromData( coverBuffer->buffer() );

        if ( !cover.isNull() )
        {
            fader->startFade( ui.coverImage->pixmap()->toImage(),
                            cover.scaled( 130, 130, Qt::IgnoreAspectRatio, Qt::SmoothTransformation ).toImage(),
                            1500 );
            return;
        }
    }

    qDebug( "Cover empty!" );

    QPixmap cover;
    cover.load( m_dataPath + "data/nocover.gif" );

    if ( minimode )
        fader->startFade( ui_mini.coverImage->pixmap()->toImage(), cover.toImage(), 1500 );
    else
        fader->startFade( ui.coverImage->pixmap()->toImage(), cover.toImage(), 1500 );
}


void
Player::skipSong()
{
    if ( minimode )
    {
        ui_mini.skipButton->setEnabled( false );
        ui_mini.loveButton->setEnabled( false );
        ui_mini.banButton->setEnabled ( false );
    }
    else
    {
        ui.skipButton->setEnabled( false );
        ui.loveButton->setEnabled( false );
        ui.banButton->setEnabled ( false );
    }

    playback->setDirectSkip();
    ws->skip();
}


void
Player::loveSong()
{
    if ( minimode )
        ui_mini.loveButton->setEnabled( false );
    else
        ui.loveButton->setEnabled( false );

    ws->love();
}


void
Player::banSong()
{
    if ( minimode )
    {
        ui_mini.skipButton->setEnabled( false );
        ui_mini.loveButton->setEnabled( false );
        ui_mini.banButton->setEnabled ( false );
    }
    else
    {
        ui.skipButton->setEnabled( false );
        ui.loveButton->setEnabled( false );
        ui.banButton->setEnabled ( false );
    }

    playback->setDirectSkip();
    ws->ban();
}


void
Player::recordToProfile( bool checked )
{
    ws->recordToProfile( checked );
}


void
Player::discoveryMode( bool checked )
{
    ws->discoveryMode( checked );
}


void
Player::setStayOnTop( bool checked )
{
    if ( checked )
        setWindowFlags( windowFlags() | Qt::WindowStaysOnTopHint );
     else
        setWindowFlags( windowFlags() ^ Qt::WindowStaysOnTopHint );

    show();
    QIcon icon( m_dataPath + "data/icon.png" );
    setWindowIcon( icon );
}


void
Player::useExternalPlayer( bool checked )
{
    if ( checked )
    {
        int externalSystem = -1;
        #ifdef WIN32
        externalSystem = 1;
        #endif
        #ifdef Q_WS_X11
        externalSystem = 2;
        #endif

        Settings::instance()->setSoundSystem( externalSystem );
    }
    else
    {
        Settings::instance()->setSoundSystem( 0 );
    }

    handshake();
}


void
Player::playbackStarting()
{
    qDebug( "Playback starting!" );

    if ( minimode )
    {
        ui_mini.stopButton->setImages( m_dataPath, "data/buttons/stop_enabled.png", "data/buttons/stop_down.png", "data/buttons/stop_hover.png", "data/buttons/stop_disabled.png" );
        ui_mini.stopButton->setEnabled( true );
    }
    else
    {
        ui.stopButton->setImages( m_dataPath, "data/buttons/stop_enabled.png", "data/buttons/stop_down.png", "data/buttons/stop_hover.png", "data/buttons/stop_disabled.png" );
        ui.stopButton->setEnabled( true );
    }
}


void
Player::playbackFinished()
{
    qDebug( "Playback finished!" );

    if ( m_timer != 0 )
    {
        m_timer->stop();
        delete m_timer;
    }
    m_timer = 0;

    if ( minimode )
    {
        ui_mini.wholeLabel->setText( "" );

        ui_mini.stopButton->setImages( m_dataPath, "data/buttons/play_enabled.png", "data/buttons/play_enabled.png", "data/buttons/play_hover.png", "data/buttons/stop_disabled.png" );
        ui_mini.skipButton->setEnabled( false );
        ui_mini.loveButton->setEnabled( false );
        ui_mini.banButton->setEnabled ( false );
        ui_mini.timeBar->setValue( 0 );
    }
    else
    {
        ui.artistLabel->setText( "" );
        ui.albumLabel->setText( "" );
        ui.trackLabel->setText( "" );
        ui.stationLabel->setText( "<font color='white'><b>Please start a station!</b></font>" );

        ui.stopButton->setImages( m_dataPath, "data/buttons/play_enabled.png", "data/buttons/play_enabled.png", "data/buttons/play_hover.png", "data/buttons/stop_disabled.png" );
        ui.skipButton->setEnabled( false );
        ui.loveButton->setEnabled( false );
        ui.banButton->setEnabled ( false );
        ui.timeBar->setValue( 0 );
    }

    loadDefaultCover();
    animationStop();
}


void
Player::webConnect()
{
    qDebug( "New WebRequest incoming!" );

    webSocket = serverSocket->nextPendingConnection();
    connect( webSocket, SIGNAL( readyRead() ), this, SLOT( webRequest() ) );
}


void
Player::webRequest()
{
    QByteArray in = webSocket->readAll();
    ws->changeStation( QString( in ) );

    delete webSocket;
}


void
Player::openAbout()
{
    AboutDialog about( this, m_dataPath );
    about.exec();
}


void
Player::openConfig()
{
    Settings::instance()->exec();
}


void
Player::openMenu()
{
    menu->exec( QCursor::pos() );
}


void
Player::openStationDialog()
{
    station->show();
}


void
Player::openUrlDialog()
{
    UrlDialog urlDialog( this, ws );
    urlDialog.exec();
}


void
Player::openTagDialog()
{
    if ( playback->isPlaying() )
    {
        TagDialog tagDialog( this, playingSong, ws );
        tagDialog.execSmall();
    }
}


void
Player::openJournalMenu()
{
    if ( playback->isPlaying() )
    {
        QMenu journalMenu( this );

        QAction *artistAction = new QAction( tr( "Write a Journal about this Artist" ), this );
        QAction *albumAction = new QAction( tr( "Write a Journal about this Album" ), this );
        QAction *trackAction = new QAction( tr( "Write a Journal about this Track" ), this );
        journalMenu.addAction( artistAction );
        journalMenu.addAction( albumAction );
        journalMenu.addAction( trackAction );

        connect( artistAction, SIGNAL( triggered() ), this, SLOT( journalArtist() ) );
        connect( albumAction, SIGNAL( triggered() ), this, SLOT( journalAlbum() ) );
        connect( trackAction, SIGNAL( triggered() ), this, SLOT( journalTrack() ) );

        journalMenu.exec( QCursor::pos() );
    }
}


void
Player::changeStationPrepared()
{
    if ( !minimode )
        ui.stationLabel->setText( "<font color='white'><b>Connecting to station, please wait...</b></font>" );
}


void
Player::changeStationResult( bool error )
{
    if ( error && !playback->isPlaying() )
        playbackFinished();

    if ( !error && !playback->isPlaying() )
    {
        QString m3uUrl = m_dataPath + "data/default.m3u";

        if ( Settings::instance()->actAsProxy() && !m_proxyConnected )
        {
            #ifdef WIN32
            WCHAR val[1024];
            memset( val, 0, 1024 );
            MultiByteToWideChar( CP_UTF8, 0, m3uUrl.toAscii(), m3uUrl.length(), val, 1024 );
            ShellExecute( 0, 0, val, NULL, NULL, SW_SHOW );
            #endif

            #ifdef Q_WS_MAC
            FSRef f;
            FSPathMakeRef( m3uUrl.toAscii().data(), &f, false );
            LSOpenFSRef( &f, NULL );

            FSRef application;
            LSGetApplicationForItem( &f, kLSRolesAll, &application, NULL );

            LSLaunchFSRefSpec launchSpec = { &application, 1, &f, NULL, kLSLaunchDefaults|kLSLaunchDontAddToRecents, NULL };
            LSOpenFromRefSpec( &launchSpec, NULL );
            #endif
        }

        int res = QMessageBox::Ok;
        #ifdef Q_WS_X11
        while ( Settings::instance()->actAsProxy() && !m_proxyConnected && res != QMessageBox::Cancel)
        {
            res = QMessageBox::information( this, "Attention", "Start your media-player with the following url, now: http://localhost:32214", QMessageBox::Ok, QMessageBox::Cancel );
        }
        #endif

        if ( !Settings::instance()->actAsProxy() || res == QMessageBox::Ok )
        {
            if ( !playback->startPlayback( streamUrl ) )
            {
                QMessageBox::information( this, "Error", "Can't open sound-device! Please make sure your settings are correct!" );
                animationStop();
            }
        }
        else
            animationStop();
    }
}


void
Player::volumeChanged( int value )
{
    playback->setVolume( value );
    Settings::instance()->setVolume( value );
}


void
Player::stopPlayback()
{
    animationStop();
    playingSong.clear();

    if ( playback->isPlaying() )
        playback->stopPlayback();
    else
        station->show();
}


void
Player::browseArtist()
{
    Settings::instance()->startBrowser( playingSong.artistUrl() );
}


void
Player::browseAlbum()
{
    Settings::instance()->startBrowser( playingSong.albumUrl() );

    return;

    delete centralWidget();

    if ( minimode )
    {
        setWindowFlags( Qt::Window );
        show();

        ui.setupUi( this );
        minimode = false;
    }
    else
    {
        setWindowFlags( Qt::Window | Qt::FramelessWindowHint );
        show();

        ui_mini.setupUi( this );
        minimode = true;
    }

    QTimer::singleShot( 0, this, SLOT( init() ) );
}


void
Player::browseTrack()
{
    Settings::instance()->startBrowser( playingSong.trackUrl() );
}


void
Player::journalArtist()
{
    Settings::instance()->startBrowser( QString( "http://www.last.fm/user/%1/journal/&action=create&artistname=%2" )
                                        .arg( Settings::instance()->username() )
                                        .arg( playingSong.artist() ) );
}


void
Player::journalAlbum()
{
    Settings::instance()->startBrowser( QString( "http://www.last.fm/user/%1/journal/&action=create&artistname=%2&albumname=%3" )
                                        .arg( Settings::instance()->username() )
                                        .arg( playingSong.artist() )
                                        .arg( playingSong.album() ) );
}


void
Player::journalTrack()
{
    Settings::instance()->startBrowser( QString( "http://www.last.fm/user/%1/journal/&action=create&artistname=%2&trackname=%3" )
                                        .arg( Settings::instance()->username() )
                                        .arg( playingSong.artist() )
                                        .arg( playingSong.track() ) );
}


void
Player::dragEnterEvent( QDragEnterEvent *event )
{
    if ( event->mimeData()->hasFormat( "text/plain" ) && event->mimeData()->text().startsWith( "lastfm://" ) )
        event->acceptProposedAction();
}


void
Player::dropEvent( QDropEvent *event )
{
    QString station = event->mimeData()->text().remove( 0, 9 ).split( "\n" )[0];
    ws->changeStation( station );

    event->acceptProposedAction();
}


void
Player::proxyClientConnected()
{
    m_proxyConnected = true;
}


void
Player::proxyClientDisconnected()
{
    m_proxyConnected = false;
}


void
Player::closeEvent( QCloseEvent *event )
{
    Settings::instance()->setPositionX( pos().x() );
    Settings::instance()->setPositionY( pos().y() );
}

